@@ -254,7 +254,7 @@ GLOBAL bool show_omitted_dirs_option;
 GLOBAL bool sparse_option;
 GLOBAL unsigned tar_sparse_major;
 GLOBAL unsigned tar_sparse_minor;
-  
+
 GLOBAL bool starting_file_option;
 
 /* Specified maximum byte length of each tape volume (multiple of 1024).  */
@@ -296,7 +296,7 @@ GLOBAL int archive;
 /* Nonzero when outputting to /dev/null.  */
 GLOBAL bool dev_null_output;
 
-/* Timestamps: */ 
+/* Timestamps: */
 GLOBAL struct timespec start_time;        /* when we started execution */
 GLOBAL struct timespec volume_start_time; /* when the current volume was
 					     opened*/
@@ -546,10 +546,6 @@ enum read_header tar_checksum (union block *header, bool silent);
 void skip_file (off_t size);
 void skip_member (void);
 
-/* Module mangle.c.  */
-
-void extract_mangle (void);
-
 /* Module misc.c.  */
 
 void assign_string (char **dest, const char *src);
@@ -727,5 +723,3 @@ bool utf8_convert (bool to_utf, char const *input, char **output);
 void set_transform_expr (const char *expr);
 bool transform_name (char **pinput);
 bool transform_name_fp (char **pinput, char *(*fun)(char *));
-     
-     
